/* ipqmpd - ip queuing multiplex daemon
 *
 *  (C) 2000 by Harald Welte <laforge@sunbeam.franken.de>
 *
 *  This code is released under the terms of GNU GPL
 *
 *  $Id: ipqmpd.h,v 1.10 2000/08/21 13:31:03 laforge Exp $
 */

#ifndef _IPQMPD_H
#define _IPQMPD_H

#include <stdio.h>
#include "queue.h"

#ifdef DEBUG
#define DEBUGP(format, args...) fprintf(stderr, format, ## args)
#define ipqmpd_error(format, args...) fprintf(stderr, format, ## args)
#else
#define DEBUGP(format, args...)
#define ipqmpd_error(format, args...) fprintf(logfile, format, ## args)
#endif

#define IPQMPD_LOGFILE	"/var/log/ipqmpd.log"

#define IPQMPD_MAX_PKTIDS	256

#define PEER_ADDR_LEN		25

/* some constants regarding sun_path */
#define IPQMPD_ADDR 		"\0ipqmpd"
#define IPQMPD_ADDR_LEN 	7
#define IPQMPD_PEER_FORMAT 	"X%s"
#define IPQMPD_DAEMON_FORMAT 	"XXXX%s"

#define IPQMPD_CTL_IBUF		1024

/* error types */
enum {
	ERRUNDEF = 0,
	ERROOM,
	ERRENQ,
	ERRFCNTL,
	ERRBIND,
	ERRCONN,
	ERRSOCK,
	ERREOF,
	ERRMAGIC,
	ERRPKTID,
	ERRQEMP,
	ERRSEND,
	ERRTRACK,
	ERRIPQ,
	ERRRECV,
};

/* valid states of a peer */
enum {
	PEER_STATE_NONE = 0,
	PEER_STATE_CONN,
	PEER_STATE_REG,
	PEER_STATE_AUTH
};

typedef struct ipqmpd_peer {
	struct ipqmpd_peer *prev;
	struct ipqmpd_peer *next;
	int		fd;
	int		ctlfd;
	u_int32_t	flags;
	u_int8_t	state;
	u_int8_t	copy_mode;
	size_t		copy_range;
	unsigned long	mark;
	unsigned long	pkt_id[IPQMPD_MAX_PKTIDS];
	char		addr[PEER_ADDR_LEN];
	queue_t *out_q;
	queue_t	*kernel_q;
	queue_t *ctl_q;
} ipqmpd_peer_t;

ipqmpd_peer_t *clients;

int hifd;

FILE *logfile;

#endif /* _IPQMPD_H */
